/*
 * Copyright (C) 2007 Javier Loureiro
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <stdlib.h>
#include <gtk/gtk.h>
#include <SDL.h>

#include <gtk-sdl.h>

static gboolean sdl_main (gpointer data)
{
  GtkSDL *pSdl = data ;
  SDL_Surface *sdlLoadedPicture1 = NULL;
  SDL_Surface *sdlLoadedPicture2 = NULL;
  int quitProgram = 0;
  signed short alpha = 0;
  signed char delta_alpha = 7;

  sdlLoadedPicture1 = SDL_LoadBMP ("gtk.bmp");
  if (sdlLoadedPicture1 == NULL)
  {
    exit (3);
  }
  sdlLoadedPicture2 = SDL_LoadBMP ("sdl.bmp");
  if (sdlLoadedPicture2 == NULL)
  {
    exit (3);
  }

  while (!quitProgram)
  {
    SDL_Event event;

    while (gtk_events_pending ())
    {
      gtk_main_iteration ();
    }

    /* Lecture des vnements dans la queue d'vnements */
    while (SDL_PollEvent (&event))
    {
      switch (event.type)
      {
        /* Evenement de fermeture d'application, on sort la boucle */
        case SDL_QUIT:
          quitProgram = 1;
        break;
      }
    }

    alpha += delta_alpha;
    if (alpha >= SDL_ALPHA_OPAQUE)
    {
      alpha = SDL_ALPHA_OPAQUE;
      delta_alpha = -delta_alpha;
    }
    else if(alpha<=0)
    {
      alpha = 0;
      delta_alpha = -delta_alpha;
    }

    /* On efface l'cran */
    SDL_BlitSurface (sdlLoadedPicture1, NULL, pSdl->surface, NULL);

    /* Mise  jour de l'alpha */
    SDL_SetAlpha (sdlLoadedPicture2, SDL_SRCALPHA, alpha);

    /* Affichage de l'image  l'cran */
    SDL_BlitSurface (sdlLoadedPicture2, NULL, pSdl->surface, NULL);

    /* Echange des deux buffers */
    /*SDL_Flip (pSdl->surface);*/
    gtk_sdl_display_update (GTK_SDL (pSdl));
  }
  SDL_FreeSurface (sdlLoadedPicture1);
  SDL_FreeSurface (sdlLoadedPicture2);
  return FALSE;
}

static void Clicked(GtkWidget *p_button, gpointer data)
{
  static gboolean run = FALSE;

  if (!run)
  {
    g_idle_add (sdl_main, data);
    gtk_button_set_label (GTK_BUTTON (p_button), "Stop");
    run = TRUE;
  }
  else
  {
    g_idle_remove_by_data (data);
    gtk_button_set_label (GTK_BUTTON (p_button), "Start");
    run = FALSE;
  }
}

static void initSDL (void)
{
  if (SDL_Init (SDL_INIT_VIDEO) < 0)
  {
    fprintf (stderr, "Erreur  l'initialisation de la SDL : %s\n",
             SDL_GetError ());
    exit (EXIT_FAILURE);
  }
  atexit (SDL_Quit);
}

int main (int argc, char **argv)
{
  GtkWidget *pSdl = NULL;
  GtkWidget *pWindow = NULL;
  GtkWidget *pButton = NULL;
  GtkWidget *pVbox = NULL;

  initSDL ();
  gtk_init (&argc, &argv);

  pButton = gtk_button_new_with_label ("Start");
  pVbox = gtk_vbox_new (FALSE, 0);

  pWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  pSdl = gtk_sdl_new (200, 200, 16, SDL_SWSURFACE | SDL_DOUBLEBUF);

  gtk_box_pack_start (GTK_BOX (pVbox), pSdl, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (pVbox), pButton, FALSE,TRUE, 0);
  gtk_container_add (GTK_CONTAINER (pWindow), pVbox);

  g_signal_connect (G_OBJECT (pButton), "clicked", G_CALLBACK (Clicked), pSdl);
  g_signal_connect (G_OBJECT (pWindow), "delete-event", gtk_main_quit, NULL);

  gtk_widget_show_all (pWindow);

  gtk_main ();

  return EXIT_SUCCESS;
}
