/*
 * Copyright (C) 2007 Javier Loureiro
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <stdlib.h>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <gtk/gtk.h>
#include <gtk-sdl.h>

static void Clicked (GtkWidget * pWindow, gpointer data)
{
  GtkSDL *pSdl = data;
  SDL_Surface *double_buffer =
    SDL_SetVideoMode (200, 100, 32, SDL_SWSURFACE);
  SDL_Surface *Phrase = NULL;
  SDL_Color TextColor;
  TTF_Font *Police;
  int i;

  TextColor.r = 255;
  TextColor.g = 0;
  TextColor.b = 0;
  Police = TTF_OpenFont ("arial.ttf", 50);
  Phrase = TTF_RenderText_Blended (Police, "bonjour", TextColor);

  for (i = 0; i < 0x0000FF; i += 0x000002)
  {

    SDL_FillRect (double_buffer, NULL, i);
    SDL_BlitSurface (Phrase, NULL, double_buffer, NULL);
    SDL_BlitSurface (double_buffer, NULL, pSdl->surface, NULL);

    gtk_sdl_display_update (GTK_SDL (pSdl) );
  }

  SDL_FreeSurface (double_buffer);
  SDL_FreeSurface (Phrase);

  (void)pWindow;
}

static void initSDL (void)
{
  if (TTF_Init() == -1)
  {
    fprintf (stderr, "Impossible d'initialiser SDL_ttf : %s", TTF_GetError());
    exit (EXIT_FAILURE);
  }
  if (SDL_Init (SDL_INIT_VIDEO) < 0)
  {
    fprintf (stderr, "Erreur  l'initialisation de la SDL : %s\n",
             SDL_GetError());
    exit (EXIT_FAILURE);
  }
}

int main (int argc, char **argv)
{
  GtkWidget *pSdl = NULL;
  GtkWidget *pWindow = NULL;
  GtkWidget *pButton = NULL;
  GtkWidget *pVbox = NULL;

  initSDL();
  gtk_init (&argc, &argv);

  pButton = gtk_button_new_with_label ("test");
  pVbox = gtk_vbox_new (TRUE, 0);
  pWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  pSdl = gtk_sdl_new (200, 100, 32, SDL_SWSURFACE);

  gtk_box_pack_start (GTK_BOX (pVbox), pSdl, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (pVbox), pButton, TRUE, TRUE, 0);
  gtk_container_add (GTK_CONTAINER (pWindow), pVbox);

  g_signal_connect (G_OBJECT (pButton), "clicked", G_CALLBACK (Clicked),
                    pSdl);
  g_signal_connect (G_OBJECT (pWindow), "destroy",
                    G_CALLBACK (gtk_main_quit), NULL);

  gtk_widget_show_all (pWindow);

  gtk_main();

  return EXIT_SUCCESS;
}

