/*
* Copyright (C) 2007 Javier Loureiro
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "gtk-sdl.h"

/* macros =================================================================== */
/* constants ================================================================ */
/* types ==================================================================== */
/* structures =============================================================== */
/* private variables ======================================================== */
/* private functions ======================================================== */
static void gtk_sdl_destroy (GtkObject *object)
{
  GtkSDLClass *klass = NULL;

  g_return_if_fail (object != NULL);
  g_return_if_fail (GTK_IS_SDL (object));

  klass = gtk_type_class (gtk_widget_get_type ());
  if (GTK_OBJECT_CLASS (klass)->destroy)
  {
    SDL_Event ev;

/*
 * FIXME : L'utilisateur peux ignorer l'evenement SDL_QuitEvent, dans ce cas il
 *         ne faut pas detruire le widget. Voir SDL_QuitRequested.
 */
    ev.type = SDL_QUIT;
    SDL_PushEvent (&ev);
    /*
     * On attend que l'vnement soit propage
     */
    /*g_usleep (0.5 * G_USEC_PER_SEC);*/
    (*GTK_OBJECT_CLASS (klass)->destroy) (object);
  }
}

static void gtk_sdl_send_configure (GtkSDL *sdl)
{
  GtkWidget *widget = NULL;
  GdkEventConfigure event;

#ifndef NDEBUG
  puts ("before sdl send configure");
#endif

  widget = GTK_WIDGET (sdl);

  event.type = GDK_CONFIGURE;
  event.window = widget->window;
  event.send_event = TRUE;
  event.x = widget->allocation.x;
  event.y = widget->allocation.y;
  event.width = widget->allocation.width;
  event.height = widget->allocation.height;

	gtk_widget_event (widget, (GdkEvent*)&event);

#ifndef NDEBUG
  puts ("after sdl send configure");
#endif
}

static void gtk_sdl_realize (GtkWidget *widget)
{
  GtkSDL *sdl = NULL;
  GdkWindowAttr attributes;
  gint attributes_mask = 0;

#ifndef NDEBUG
  puts ("before sdl realize");
#endif

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_SDL (widget));

  sdl = GTK_SDL (widget);
  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);

  attributes.window_type = GDK_WINDOW_CHILD;
  attributes.x = widget->allocation.x;
  attributes.y = widget->allocation.y;
  attributes.width = widget->allocation.width;
  attributes.height = widget->allocation.height;
  attributes.wclass = GDK_INPUT_OUTPUT;
  attributes.visual = gtk_widget_get_visual (widget);
  attributes.colormap = gtk_widget_get_colormap (widget);
  attributes.event_mask = gtk_widget_get_events (widget) | GDK_EXPOSURE_MASK;

  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;

	widget->window = gdk_window_new (gtk_widget_get_parent_window (widget),
                                   &attributes, attributes_mask);
	gdk_window_set_user_data (widget->window, sdl);

  widget->style = gtk_style_attach (widget->style, widget->window);
  gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);

  gtk_sdl_send_configure (GTK_SDL (widget));

#ifndef NDEBUG
  puts ("after sdl realize");
#endif
}

static void gtk_sdl_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
{
#ifndef NDEBUG
  puts ("before sdl size allocate");
#endif

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_SDL (widget));
  g_return_if_fail (allocation != NULL);

  widget->allocation = *allocation;
  /* FIXME, TODO-1.3: back out the MAX() statements */
  widget->allocation.width = MAX (1, widget->allocation.width);
  widget->allocation.height = MAX (1, widget->allocation.height);

	if (GTK_WIDGET_REALIZED (widget))
	{
		gdk_window_move_resize (widget->window, allocation->x, allocation->y,
                            allocation->width, allocation->height);

    gtk_sdl_send_configure (GTK_SDL (widget));
  }

#ifndef NDEBUG
  puts ("after sdl size allocate");
#endif
}

static void gtk_sdl_surface_attach (GtkSDL *sdl)
{
  gchar SDL_windowhack[32];

#ifndef NDEBUG
  puts ("before sdl surface attach");
  /* Attach the SDL_Surface */
  puts ("attaching the surface");
#endif

#ifdef G_OS_WIN32
  sprintf (SDL_windowhack, "SDL_WINDOWID=%ld",
           GDK_WINDOW_HWND (GTK_WIDGET (sdl)->window));
#else
  sprintf (SDL_windowhack, "SDL_WINDOWID=%ld",
           GDK_WINDOW_XWINDOW (GTK_WIDGET (sdl)->window));
#endif

#ifndef NDEBUG
  puts (SDL_windowhack);
#endif

  putenv (SDL_windowhack);
  SDL_QuitSubSystem (SDL_INIT_VIDEO);

#ifndef NDEBUG
  puts ("before creating new surface");
#endif

  if (SDL_InitSubSystem (SDL_INIT_VIDEO) < 0)
  {
    fprintf (stderr, "unable to init SDL: %s", SDL_GetError ());
    return ;
  }
#ifndef NDEBUG
  puts ("after creating new surface");
#endif

  if (sdl->flags &= (SDL_OPENGLBLIT | SDL_DOUBLEBUF))
  {
    SDL_GL_SetAttribute (SDL_GL_DOUBLEBUFFER, 1);
  }

  if (sdl->surface != NULL)
  {
#ifndef NDEBUG
    puts ("TODO: deallocate previus surface");
#endif

  }

  sdl->surface = SDL_SetVideoMode (sdl->width, sdl->height, sdl->bpp, sdl->flags);
  if (sdl->surface == NULL)
  {
    fprintf (stderr, "Unable to set the video mode: %s", SDL_GetError());
    return ;
  }

#ifndef NDEBUG
  puts ("after sdl surface attach");
#endif
}

static gint gtk_sdl_expose (GtkWidget *widget, GdkEventExpose *event)
{
  SDL_Surface *surface = NULL;
#if 0
  SDL_Rect dest;
#endif

#ifndef NDEBUG
  puts ("before sdl expose");
#endif

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_SDL (widget), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);

  if (event->count > 0)
  {
    return FALSE;
  }
  surface = SDL_GetVideoSurface ();
  if (surface == NULL)
  {
    gtk_sdl_surface_attach (GTK_SDL (widget));
  }
  gtk_sdl_display_update (GTK_SDL(widget));

#if 0
  if (surface != NULL)
  {
#ifndef NDEBUG
    puts ("update rects from expose");
#endif

    dest.x = event->area.x;
    dest.y = event->area.y;
    dest.w = event->area.width;
    dest.h = event->area.height;

    /* ADDED NOW! */
    SDL_UpdateRect (surface, 0, 0, 0, 0);

    /* LATER OFF */
    SDL_UpdateRects (surface, 1, &dest);
  }
  else
  {
    gtk_sdl_surface_attach (GTK_SDL (widget));
  }
#endif

#ifndef NDEBUG
  puts ("after sdl expose");
#endif

  surface = SDL_GetVideoSurface ();

#ifndef NDEBUG
  printf ("surface is : %p\n", (void *)surface);
#endif
#if 0
  SDL_UpdateRect (surface, 0, 0, 0, 0);
  puts ("rectangle updated");
#endif

  return FALSE;
}

static void gtk_sdl_class_init (GtkSDLClass *class)
{
  GtkObjectClass *object_class = NULL;
  GtkWidgetClass *widget_class = NULL;

#ifndef NDEBUG
  puts ("before class init");
#endif

  widget_class = GTK_WIDGET_CLASS (class);
  widget_class->realize = gtk_sdl_realize;
  widget_class->size_allocate = gtk_sdl_size_allocate;
  widget_class->expose_event = gtk_sdl_expose;

  object_class = GTK_OBJECT_CLASS (class);
  object_class->destroy = gtk_sdl_destroy;

#if 0
  widget_class->size_request = gtk_dial_size_request;
  widget_class->size_allocate = gtk_dial_size_allocate;
#endif

#ifndef NDEBUG
  puts ("after class init");
#endif
}

static void gtk_sdl_init (GtkSDL *sdl)
{
#ifndef NDEBUG
  puts ("before sdl init");
#endif

	GTK_WIDGET_SET_FLAGS (sdl, GTK_CAN_FOCUS);
	if ( GTK_WIDGET_CAN_FOCUS(sdl))
	{
		gtk_widget_grab_focus (GTK_WIDGET (sdl));
	}
	gtk_widget_set_events (GTK_WIDGET (sdl), GDK_KEY_PRESS_MASK);

#ifndef NDEBUG
  puts ("after sdl init");
#endif
}
/* internal public functions ================================================ */
G_DEFINE_TYPE (GtkSDL, gtk_sdl, GTK_TYPE_WIDGET)

/* entry points ============================================================= */
/**
 * gtk_sdl_new:
 *
 * @sdl:
 * @width:
 * @height:
 * @bpp:
 * @flags :
 *
 * Returns:
 */
GtkWidget *gtk_sdl_new (gint width, gint height, gint bpp, Uint32 flags)
{
  GtkSDL *sdl = NULL;

#ifndef NDEBUG
	g_print ("before sdl new");
#endif

  sdl = gtk_type_new (gtk_sdl_get_type ());

  sdl->width = width;
  sdl->height = height;
  sdl->bpp = bpp;
  sdl->flags = flags;

	gtk_sdl_size (GTK_SDL (sdl), width, height);

#if 0
	timeout_id = gtk_timeout_add ((33*10)/10, cbDrawAreaUpdate, (gpointer)darea);
	gtk_object_set_data (GTK_OBJECT (sdl), "timeout_id", (gpointer)timeout_id);
#endif

#ifndef NDEBUG
  puts ("after sdl new");
#endif

  return GTK_WIDGET (sdl);
}

/**
 * gtk_sdl_size:
 *
 * @sdl:
 * @width:
 * @height :
 */
void gtk_sdl_size (GtkSDL *sdl, gint width, gint height)
{
#ifndef NDEBUG
  puts ("before sdl size");
#endif

  g_return_if_fail (GTK_IS_SDL (sdl));

  GTK_WIDGET (sdl)->requisition.width = width;
  GTK_WIDGET (sdl)->requisition.height = height;

  gtk_widget_queue_resize (GTK_WIDGET (sdl));

#ifndef NDEBUG
  puts ("after sdl size");
#endif
}

/**
 * gtk_sdl_display_update:
 *
 * @sdl:
 */
void gtk_sdl_display_update (GtkSDL *sdl)
{

#ifndef NDEBUG
  puts ("before sdl display update");
#endif

  g_return_if_fail (sdl != NULL);
  g_return_if_fail (GTK_IS_SDL (sdl));
  g_return_if_fail (sdl->surface);

  if ((sdl->flags &= SDL_OPENGL) || (sdl->flags &= SDL_OPENGLBLIT))
  {
#if 0
    if (dkActiveUpdaterGL)
    {
      (*dkActiveUpdaterGL) ();
    }
#endif
    SDL_GL_SwapBuffers ();
  }
  else
  {
#if 0
    if (dkActiveUpdater)
    {
      (*dkActiveUpdater)();
    }
#endif

#ifndef NDEBUG
    puts ("flip video surface");
#endif

    SDL_Flip (sdl->surface);
  }

#ifndef NDEBUG
  puts ("before update rect");
#endif

  SDL_UpdateRect (sdl->surface, 0, 0, 0, 0);

#ifndef NDEBUG
  puts ("after update rect");
  puts ("after display update");
#endif
}

/* public variables ========================================================= */
