/*
* Copyright (C) 2007 BEYLER Jean Christophe
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
*/

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <SDL.h>
#include <gtk/gtk.h>
#include <gtk-sdl.h>

#include "HandleImage.h"

/* macros =================================================================== */
/* constants ================================================================ */
#define WIDTH 640
#define HEIGHT 480

/* types ==================================================================== */
/* structures =============================================================== */
/* private variables ======================================================== */
SHandleImage *hi = NULL;
SDL_Surface *screen = NULL;

/* private functions ======================================================== */

static void Stop (void)
{
  Handle_Destroy (hi);
}

static int Init (int areaw, int areah)
{
  int i;
  SDL_Rect r;

  /* Random */
  srand ((unsigned int)time (NULL));

  r.x = 0;
  r.y = 0;
  r.w = areaw;
  r.h = areah;

  /* Gestion du handler */
  hi = Handle_Init ("gtksdl.bmp", &r);
  if (hi == NULL)
  {
    return EXIT_FAILURE;
  }
  for (i = 0;i < 10;i++)
  {
    Handle_AddImage (hi);
  }
  return EXIT_SUCCESS;
}

static void Frame (SDL_Surface *sc)
{
  /* Gestion de l'image */
  SDL_FillRect (sc, NULL, SDL_MapRGB (sc->format, 0, 0, 0));
  Handle_Frame (hi, sc);
  SDL_Flip (sc);
}

static void initSDL (void)
{
  if (SDL_Init (SDL_INIT_VIDEO) < 0)
  {
    fprintf (stderr, "Erreur a l'initialisation de la SDL : %s\n",
             SDL_GetError ());
    exit (EXIT_FAILURE);
  }
  atexit (SDL_Quit);
}

static void refresh (gpointer data)
{
  GtkSDL *p_gtk_sdl = NULL;

  g_return_if_fail (data != NULL);
  g_return_if_fail (GTK_IS_SDL (data));

  p_gtk_sdl = GTK_SDL (data);
  gtk_sdl_display_update (p_gtk_sdl);
  while (gtk_events_pending ())
  {
    gtk_main_iteration ();
  }
}

static int sdl_main (GtkSDL *p_gtk_sdl)
{
  int ret = EXIT_FAILURE;
  SDL_Event event;
  int done = 0;
  int last, now;
  int fps;

  g_return_val_if_fail (p_gtk_sdl != NULL, ret);
  g_return_val_if_fail (GTK_IS_SDL (p_gtk_sdl), ret);

  screen = p_gtk_sdl->surface;
  if (screen != NULL)
  {
    if (Init (screen->w, screen->h) == EXIT_SUCCESS)
    {
      last = SDL_GetTicks ();
      fps = 0;
      /* Boucle generale */
      while (!done)
      {
        refresh (p_gtk_sdl);
        Frame (screen);
        fps++;
        now = SDL_GetTicks ();
        if (now - last > 1000)
        {
          fps /= (now - last) / 1000;
          printf ("FPS : %d\n", fps);
          fps = 0;
          last = SDL_GetTicks ();
        }
        /* Traiter les evenements */
        while (SDL_PollEvent (&event))
        {
          switch (event.type)
          {
            /* Si on veut quitter, done=1 suffira */
            case SDL_QUIT:
              done = 1;
            break;
            /* Si on vient de relacher une touche de clavier */
            case SDL_KEYUP:
              /* Et si c'est la lettre q */
              /*
               * Attention, ceci ne fonctionne pas avec tous les Systemes
               * d'exploitation et tous les claviers.
               * Pour le rendre portable, il faudrait passer par les Unicodes,
               * mais cela sort du cadre de ce tutoriel d'introduction.
               * Si vous voulez utiliser la lettre 'q' pour quitter et ceci ne
               * fonctionne pas, alors c'est la lettre 'a' qu'il faudra utiliser
               * (c'est la place de la lettre 'q' sur le clavier QWERTY).
               * Si vous voulez vraiment la lettre 'q', remplacer SDLK_q par
               * SDLK_a, cela devrait faire l'affaire.
               */
              if (event.key.keysym.sym == SDLK_q)
              {
                /* On met done a 1 pour quitter */
                done = 1;
              }
            break;
          }
        }
      }
      Stop();
      SDL_Quit();
      ret = EXIT_SUCCESS;
    }
  }
  return ret;
}

static void cb_clicked (GtkWidget *p_widget, gpointer user_data)
{
  gtk_widget_set_sensitive (p_widget, FALSE);
  sdl_main (GTK_SDL (user_data));
  gtk_widget_set_sensitive (p_widget, TRUE);
}

/* internal public functions ================================================ */
/* entry points ============================================================= */
int main (int argc, char **argv)
{
  GtkWidget *p_gtk_sdl = NULL;
  GtkWidget *p_window = NULL;
  GtkWidget *p_button = NULL;
  GtkWidget *p_vbox = NULL;

  initSDL ();
  gtk_init (&argc, &argv);

  p_button = gtk_button_new_with_label ("Start");
  p_vbox = gtk_vbox_new (FALSE, 0);

  p_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  p_gtk_sdl = gtk_sdl_new (WIDTH, HEIGHT, 32, SDL_DOUBLEBUF | SDL_HWSURFACE);
  screen = GTK_SDL (p_gtk_sdl)->surface;

  gtk_box_pack_start (GTK_BOX (p_vbox), p_gtk_sdl, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (p_vbox), p_button, FALSE, TRUE, 0);
  gtk_container_add (GTK_CONTAINER (p_window), p_vbox);

  g_signal_connect (G_OBJECT (p_button), "clicked", G_CALLBACK (cb_clicked),
                    p_gtk_sdl);
  g_signal_connect (G_OBJECT (p_window), "delete-event", gtk_main_quit, NULL);
  gtk_widget_show_all (p_window);

  gtk_main ();
  return EXIT_SUCCESS;
}

/* public variables ========================================================= */
